module Iup

  # A Canvas is a working area for the application.
  # 
  # Many methods working on the Canvas are drawn from the CD library:
  # http://webserver2.tecgraf.puc-rio.br/cd/
  #
  # In this example, notice how the dialog is mapped, the canvas initialised,
  # and finally the dialog is shown. 
  #
  #   mainloop do
  #     cnv = Canvas.new do
  #       rastersize '300x200'
  #       action ->(x, y){ # called on redraw: (x,y) refer to scrollbar positions, if used
  #         clear
  #         foreground CD_BLUE
  #         box 10, 100, 10, 100
  #         foreground CD_RED
  #         rectangle 10, 100, 10, 100
  #         text 200, 180, 'hello from Ruby'
  #         DEFAULT
  #       }
  #     end
  #   
  #     dlg = Dialog.new VBox.new(cnv){ margin '10x10' } do
  #       title 'IupCanvas + Canvas Draw'
  #       size '350x220'
  #     end.map
  #   
  #     cnv.init # this has to be done after mapping the dialog
  #   
  #     dlg.show
  #   end
  #   
  #
  # == Attributes
  #
  # canfocus:: Enables the control to gain focus. Values 'yes' / 'no'.
  # cursor:: Defines the mouse shape / cursor for the canvas.
  # drawsize:: Size of the drawing area in pixels, as "widthxheight".
  # expand:: Allows canvas to fill available space in indicated direction.
  #          Values 'no' / 'horizontal' / 'vertical' / 'yes'.
  # padding:: Margin in x and y directions, value as "mxn".
  # position:: <b>read-only</b> returns position in pixels within client window
  #            as "x,y".
  # rastersize:: Size of the canvas, in pixels, value as "widthxheight".
  # screenposition:: <b>read-only</b> returns position in pixels on screen
  # scrollbar:: Selects 'no' / 'horizontal' / 'vertical' / 'yes' (for both) scrollbars.
  # tip:: Tooltip string.
  #
  class Canvas < Widget
    include DragDropAttributes
    include ScrollBarAttributes

    # Creates an instance of the canvas.
    # handle:: an optional Canvas handle.
    # block:: optional block to set up Canvas attributes.
    def initialize handle = IupLib.IupCanvas(''), &block
      @handle = handle

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end

    # This must be called after the widget is mapped, to create and
    # setup the canvas.
    def init
      @canvas = CdLib.cdCreateCanvas CdLib.cdContextIup, @handle
    end

    # -- attributes

    define_attribute :border
    define_attribute :canfocus
    define_attribute :cursor
    define_attribute :drawsize
    define_attribute :expand
    define_attribute :padding
    define_readonly :position
    define_attribute :rastersize
    define_readonly :screenposition
    define_attribute :tip

    # -- CD method calls

    # --- initialisation

    # call when canvas no longer needed.
    def kill
      CdLib.cdKillCanvas @canvas
    end

    # call to activate canvas. 
    # (Not usually needed - depends on driver.)
    def activate
      CdLib.cdCanvasActivate @canvas
    end
    
    # call to deactivate canvas. 
    # (Not usually needed - depends on driver.)
    def deactivate
      CdLib.cdCanvasDeactivate @canvas
    end

    # Used to set coordinates of a viewport (the canvas coordinates).
    def viewport x1, y1, x2, y2
      CdLib.wdCanvasViewport @canvas, x1, y1, x2, y2
    end

    # Used to set coordinates of a window (the world coordinates).
    def window x1, y1, x2, y2
      CdLib.wdCanvasWindow @canvas, x1, y1, x2, y2
    end

    # redraws the underlying canvas.
    def redraw
      IupLib.IupRedraw @handle
    end

    # --- general attributes

    # Sets the foreground colour, if a value is given, else returns current value.
    #
    # Colour may be one of:
    # * string, in form "RR GG BB"
    # * colour constant, e.g. CD_BLACK, as listed in Iup
    def foreground colour=CD_QUERY
      case colour
      when String # assume in form 'RR GG BB'
        r, g, b = colour.split(' ').collect &:to_i
        CdLib.cdCanvasForeground @canvas, CdLib.cdEncodeColor(r, g, b)
      else
        CdLib.cdCanvasForeground @canvas, colour
      end
    end

    # Sets the background colour, if a value is given, else returns current value.
    #
    # Colour may be one of:
    # * string, in form "RR GG BB"
    # * colour constant, e.g. CD_BLACK, as listed in Iup
    def background colour=CD_QUERY
      case colour
      when String # assume in form 'RR GG BB'
        r, g, b = colour.split(' ').collect &:to_i
        CdLib.cdCanvasBackground @canvas, CdLib.cdEncodeColor(r, g, b)
      else
        CdLib.cdCanvasBackground @canvas, colour
      end
    end

    # Sets the writing mode for all primitives, if a value is given. 
    # Else returns the current value. 
    # See Iup for a list of available writemode values.
    def writemode mode=CD_QUERY
      CdLib.cdCanvasWriteMode @canvas, mode
    end

    # --- control
  
    # Clears the canvas.
    def clear
      CdLib.cdCanvasClear @canvas
    end


    # --- begin/end blocks

    # Begin the definition of a sequence of vertices.
    # Valid polygon mode values are listed in Iup.
    def begin_block mode
      CdLib.cdCanvasBegin @canvas, mode
    end

    # Specify a vertex at position (x, y).
    def vertex x, y
      CdLib.cdCanvasVertex @canvas, x, y
    end

    # End the definition of a sequence of vertices.
    def end_block 
      CdLib.cdCanvasEnd @canvas
    end

    # Specify action for next point on path.
    # Valid path actions are listed in Iup.
    def pathset action
      CdLib.cdCanvasPathSet @canvas, action
    end

    # --- marks

    # Configures pixel (x, y) with colour.
    def pixel x, y, colour
      CdLib.cdCanvasPixel @canvas, x, y, colour
    end

    # Draws a mark at position (x, y).
    def mark x, y
      CdLib.cdCanvasMark @canvas, x, y
    end

    # Sets the marktype, if a value is given, else returns the current value.
    # See Iup for available marktype values.
    def marktype type=CD_QUERY
      CdLib.cdCanvasMarkType @canvas, type
    end

    # Sets the marksize, if a pixel value is given,
    # else, returns the current value.
    def marksize size=CD_QUERY
      CdLib.cdCanvasMarkSize size
    end

    # --- lines

    # Draws a line from (x1, y1) to (x2, y2).
    def line x1, y1, x2, y2
      CdLib.cdCanvasLine @canvas, x1, y1, x2, y2
    end

    # Draws a rectangle bounding (xmin, ymin) to (xmax, ymax).
    def rectangle xmin, xmax, ymin, ymax
      CdLib.cdCanvasRect @canvas, xmin, xmax, ymin, ymax
    end

    # Draws an arc of an ellipse - a pie-shaped slice.
    # Angles are in degrees, counterclockwise.
    def arc xc, yc, w, h, angle1, angle2
      CdLib.cdCanvasArc @canvas, xc, yc, w, h, angle1, angle2
    end

    # Sets linestyle, if a value is given, else returns the current style.
    # See Iup for available linestyle values.
    def linestyle style=CD_QUERY
      CdLib.cdCanvasLineStyle @canvas, style
    end

    # Sets linewidth, if a pixel value is given, else returns the current width.
    def linewidth width=CD_QUERY
      CdLib.cdCanvasLineWidth @canvas, width
    end
    
    # Sets linejoin, if a value is given, else returns the current value.
    # See Iup for available linejoin values.
    def linejoin style=CD_QUERY
      CdLib.cdCanvasLineJoin @canvas, style
    end

    # Sets linecap, if a value is given, else returns the current value.
    # See Iup for available linecap values.
    def linecap style=CD_QUERY
      CdLib.cdCanvasLineCap @canvas, style
    end

    # -- filled areas

    # Fills a rectangle, according to current interior style.
    def box xmin, xmax, ymin, ymax
      CdLib.cdCanvasBox @canvas, xmin, xmax, ymin, ymax
    end

    # Fills a sector, according to current interior style.
    # A sector is an arc of an ellipse - a pie-shaped slice.
    # Angles are in degrees, counterclockwise.
    def sector xc, yc, w, h, angle1, angle2
      CdLib.cdCanvasSector @canvas, xc, yc, w, h, angle1, angle2
    end

    # Fills a chord, according to current interior style.
    # A chord is an arc of an ellipse - with the arc points connected.
    # Angles are in degrees, counterclockwise.
    def chord xc, yc, w, h, angle1, angle2
      CdLib.cdCanvasChord @canvas, xc, yc, w, h, angle1, angle2
    end

    # Sets the opacity, if a value is given, else returns current value.
    # See Iup for background opacity mode values.
    def backopacity opacity=CD_QUERY
      CdLib.cdCanvasBackOpacity @canvas, opacity
    end

    # Sets the fill mode, if a value is given, else returns current value.
    # See Iup for fill mode values.
    def fillmode mode=CD_QUERY
      CdLib.cdCanvasFillMode @canvas, mode
    end

    # Sets the interior style, if a value is given, else returns current value.
    # See Iup for interior style values.
    def interiorstyle style=CD_QUERY
      CdLib.cdCanvasInteriorStyle @canvas, style
    end

    # Sets the hatch style, if a value is given, else returns current value.
    # See Iup for hatch style values.
    def hatch style=CD_QUERY
      CdLib.cdCanvasHatch @canvas, style
    end

    # fgbg is a string representing the stipple pattern in binary
    def stipple w, h, fgbg
      CdLib.cdCanvasStipple @canvas, w, h, fgbg
    end

    # -- text

    # Draw text str at position (x, y). Expects an ANSI-string.
    def text x, y, str
      CdLib.cdCanvasText @canvas, x, y, str
    end

    # Sets text font.
    # typeface:: 'Courier', 'Helvetica', 'Times' built-in, but any known 
    #            font can be used
    # style:: see Iup for list of text styles.
    # size:: the font height, default is 12.
    def font typeface, style, size
      CdLib.cdCanvasFont @canvas, typeface, style, size
    end

    # Sets the current font.
    def nativefont font
      CdLib.cdCanvasNativefont @canvas, font
    end

    # Sets vertical and horizontal alignment if a value is given, 
    # else returns current value.
    # See Iup for text alignment values.
    def textalignment alignment=CD_QUERY
      CdLib.cdCanvasTextAlignment @canvas, alignment
    end

    # Sets orientation angle if a value is given, 
    # else returns current value.
    def textorientation orientation=CD_QUERY
      CdLib.cdCanvasTextOrientation @canvas, orientation
    end

    # -- vector text

    # Draws str as vector text at position (x, y).
    # Respects 
    def vectortext x, y, str
      CdLib.cdCanvasVectorText @canvas, x, y, str
    end

    # Defines the text direction using two points: (x1, y1) and (x2, y2).
    def vectortext_direction x1, y1, x2, y2
      CdLib.cdCanvasVectorTextDirection @canvas, x1, y1, x2, y2
    end

    # Modifies font size so str fits within a box width x height in size.
    def vectortext_size width, height, str
      CdLib.cdCanvasVectorTextSize @canvas, width, height, str
    end

    # If a size is given, then the font size is given that height.
    # Otherwise, the current value is returned.
    def vectortext_charsize size=CD_QUERY
      CdLib.cdCanvasVectorCharSize @canvas, size
    end

    # Directly modifies the font size.
    def vectortext_fontsize size_x, size_y
      CdLib.cdCanvasVectorFontSize @canvas, size_x, size_y
    end

    # Replaces current vector font with that loaded from given filename.
    # Returns the font name, or null.
    def vectortext_loadfont filename
      CdLib.cdCanvasVectorFont @canvas, filename
    end

    # -- callbacks

    # Action generated when the canvas needs to be redrawn.
    # Action takes a callback which accepts 2 arguments (posx, posy).
    # posx, posy are the position of the horizontal and vertical thumbs of the scrollbar.
    def action callback
      unless callback.arity == 2
        raise ArgumentError, 'action must take 2 arguments: (posx, posy)'
      end
      cb = Proc.new do |ih, posx, posy|
        callback.call posx, posy
      end
      define_callback cb, 'ACTION', :ff_i
    end
 
    include ButtonCallback

    # Called when the canvas gets or loses the keyboard focus.
    # Callback takes a single parameter: (focus)
    # focus:: non-zero if canvas gaining focus, else zero.
    def focus_cb callback
      unless callback.arity == 1
        raise ArgumentError, 'focus_cb callback must take 1 argument, the focus'
      end
      cb = Proc.new do |ih, focus|
        callback.call focus
      end
      define_callback cb, 'FOCUS_CB', :i_i
    end

    # Action generated when a key is pressed or released.
    # keypress_cb takes a 2-argument callback: (character, pressed).
    # pressed == 1 if key is pressed, 
    # pressed == 0 if key is released.
    def keypress_cb callback
      unless callback.arity == 2
        raise ArgumentError, 'keypress_cb callback must take 2 arguments: (char, press)'
      end
      cb = Proc.new do |ih, char, press|
        callback.call char, press
      end
      define_callback cb, 'KEYPRESS_CB', :ii_i
    end

    # Action generated when the mouse is moved.
    # Callback takes 3 arguments: (x, y, state)
    # x:: x position of mouse
    # y:: y position of mouse
    # state:: status of mouse buttons and certain keyboard keys at the moment the event was generated.
    #--
    # TODO: include functions, as in button_cb
    #
    def motion_cb callback
      unless callback.arity == 3
        raise ArgumentError, 'motion_cb callback must take 3 arguments: (x, y, state)'
      end
      cb = Proc.new do |ih, x, y, state|
        callback.call x, y, state
      end
      define_callback cb, 'MOTION_CB', :iis_i
    end

    # Action generated when the canvas size is changed.
    # resize_cb a 2-argument callback: (width, height).
    # width:: internal width of canvas (client width)
    # height:: internal height of canvas (client height)
    def resize_cb callback
      unless callback.arity == 2
        raise ArgumentError, 'resize_cb callback must take 2 arguments: (width, height)'
      end
      cb = Proc.new do |ih, width, height|
        callback.call width, height
      end
      define_callback cb, 'RESIZE_CB', :ii_i
    end

    # Action generated when the mouse wheel is rotated.
    # wheel_cb a 4-argument callback: (delta, x, y, status).
    # delta:: the amount the wheel was rotated in notches.
    # x, y:: position in the canvas where the event has occurred, in pixels.
    # status:: status of mouse buttons and certain keyboard keys at the moment the event was generated.
    def wheel_cb callback
      unless callback.arity == 4
        raise ArgumentError, 'wheel_cb callback must take 4 arguments: (delta, x, y, status)'
      end
      cb = Proc.new do |ih, delta, x, y, status_ptr|
        status = FFI::Pointer.new(status_ptr).read_string
        callback.call delta, x, y, status
      end
      define_callback cb, 'WHEEL_CB', :fiis_i
    end
  end
end
