module Iup

  # A Dialog is a top-level window, containing a single widget.
  #
  # == Attributes
  #
  # background:: RGB colour for background of dialog and children.
  # border:: If set, displays a resize border around dialog.
  # clientsize:: <b>read-only</b>, returns current size of box as "widthxheight".
  # cursor:: Defines the mouse shape / cursor for the dialog.
  # expand:: Allows container to fill available space in indicated direction.
  #          Values 'no' / 'horizontal' / 'vertical' / 'yes'.
  # fullscreen:: 'no' / 'yes'.
  # maxbox:: 'no' / 'yes', to show maximize box on dialog.
  # menubox:: 'no' / 'yes', to show system menu box on dialog.
  # minbox:: 'no' / 'yes', to show minimize box on dialog.
  # modal:: <b>read-only</b> Returns modal state of dialog.
  # parentdialog:: Sets/retrieves dialog to use as parent of this dialog.
  # placement:: 'normal' / 'maximized' / 'minimized' / 'full'.
  # rastersize:: Size of the dialog, in pixels, value as "widthxheight".
  # resize:: 'no' / 'yes'.
  # shrink:: 'no' / 'yes', allows dialog's children to reduce their size if
  #          the dialog is made smaller.
  # screenposition:: <b>read-only</b> returns position in pixels on screen
  #                  as "x,y".
  # size:: Size of the dialog, in character units, value as "widthxheight".
  #        Can also use: 'full' / 'half' / 'third' / 'quarter' / 'eighth'.
  # startfocus:: Name of widget to gain focus when dialog first shown.
  # tip:: Tooltip string.
  # title:: text displayed as dialog title.
  #
  class Dialog < Widget
    include DragDropAttributes

    include AttributeReference

    # Creates a new dialog for given widget.
    # widget:: the child widget to display.
    # block:: optional block to set up attributes.
    def initialize widget, &block
      @handle = IupLib.IupDialog widget.handle

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given? 
    end

    # Hides the dialog from view.
    def hide
      IupLib.IupHide @handle
    end

    # Lays out the child widget and its contents, without showign the dialog.
    def map
      IupLib.IupMap @handle
      self
    end

    # Shows the dialog at position (x, y).
    def popup x, y
      IupLib.IupPopup @handle, x.to_i, y.to_i
    end
    
    # Shows the dialog at the default screen position, 
    # or at (x, y) if specified.
    def show x=nil, y=nil
      if x.nil? and y.nil?
        IupLib.IupShow @handle
      else
        IupLib.IupShowXY @handle, x.to_i, y.to_i
      end
    end

    # -- attributes

    # --- common

    define_attribute :background
    define_attribute :border
    define_readonly :clientsize

    def cursor image=nil # :nodoc:
      attribute_reference 'CURSOR', ImageWidget, image
    end

    define_attribute :expand
    define_attribute :rastersize
    define_readonly :screenposition
    define_attribute :tip
    define_attribute :title

    # --- exclusive

    # Identifies button to activate when 'enter' is pressed.
    # Leave item nil to retrieve current button.
    def defaultenter item=nil
      attribute_reference 'DEFAULTENTER', Button, item
    end

    # Identifies button to activate when 'escape' is pressed.
    # Leave item nil to retrieve current button.
    def defaultesc item=nil
      attribute_reference 'DEFAULTESC', Button, item
    end

    define_attribute :fullscreen

    # Defines or retrieves icon to use for dialog.
    # item:: optional instance of ImageWidget
    def icon item=nil
      attribute_reference 'ICON', ImageWidget, item
    end

    define_attribute :maxbox

    # Assigns given item as the menu for this dialog.
    # item may be an instance of Menu, or the string name for a menu.
    def menu item=nil
      attribute_reference 'MENU', Menu, item
    end

    define_attribute :menubox
    define_attribute :minbox
    define_readonly :modal

    def parentdialog parent=nil # :nodoc:
      attribute_reference 'PARENTDIALOG', Dialog, parent
    end

    define_attribute :placement
    define_attribute :resize
    define_attribute :shrink

    def startfocus item=nil # :nodoc:
      attribute_reference 'STARTFOCUS', Widget, item
    end

    # -- callbacks
    
    # Called right before the dialog is closed.
    def close_cb callback
      unless callback.arity.zero?
        raise ArgumentError, 'close_cb callback must take 0 arguments'
      end
      cb = Proc.new do |ih|
        callback.call
      end
      define_callback cb, 'CLOSE_CB', :plain
    end

     # Action generated when the canvas size is changed.
    # resize_cb a 2-argument callback: (width, height).
    # width:: internal width of canvas (client width)
    # height:: internal height of canvas (client height)
    def resize_cb callback
      unless callback.arity == 2
        raise ArgumentError, 'resize_cb callback must take 2 arguments: (width, height)'
      end
      cb = Proc.new do |ih, width, height|
        callback.call width, height
      end
      define_callback cb, 'RESIZE_CB', :ii_i
    end

    # Called right after the dialog is shown, hidden, maximized, minimized or restored from minimized/maximized. 
    # Callback takes one argument, the state of the change.
    def show_cb callback
      unless callback.arity == 1
        raise ArgumentError, 'show_cb callback must take 1 argument: (state)'
      end
      cb = Proc.new do |ih, state|
        callback.call state
      end
      define_callback cb, 'SHOW_CB', :i_i
    end
  end
end
