module Iup

  # Creates and shows an AlarmDialog, returning the number of the button clicked.
  #
  #   mainloop do
  #     case alarm 'IupAlarm Example', 'File not saved!  Save it now?', 'Yes', 'No', 'Cancel'
  #     when 1
  #       message 'Save file', 'File saved successfully - leaving program'
  #     when 2
  #       message 'Save file', 'File not saved - leaving program anyway'
  #     when 3
  #       message 'Save file', 'Operation cancelled'
  #     end
  #     exit # needed, as we don't have a persistent dialog, so don't wish to run the   mainloop
  #   end
  #
  # title:: the title of the dialog window
  # msg:: the text to display within the dialog
  # button1:: label for the first button
  # button2:: label for the optional second button
  # button3:: label for the optional third button
  #
  def alarm title, msg, button1, button2=nil, button3=nil
    IupLib.IupAlarm title, msg, button1, button2, button3
  end

  # Shows a modal dialog allowing user to select a colour.  
  # As input, provide the x,y coordinates to show dialog.  
  # Returns r/g/b triple of selected colour, or nil if "cancel" clicked.
  #
  #   r, g, b = get_colour 150, 150
  # 
  def get_colour x=0, y=0
    r = FFI::MemoryPointer.new(:int, 1, 0)
    g = FFI::MemoryPointer.new(:int, 1, 0)
    b = FFI::MemoryPointer.new(:int, 1, 0)
    code = IupLib.IupGetColor x, y, r, g, b
    if code == 0
      return nil, nil, nil
    else
      return r.read_int, g.read_int, b.read_int
    end
  end

  # Convenience function to show a modal dialog to select a filename.
  # As input, accepts a path and filter for the file.  
  # Returns the status code and selected filename.
  # 
  # Example, selecting a filename ending in ".txt" from current directory.
  #
  #   code, file = get_file './*.txt'
  # 
  def get_file filter=''
    file = ' '*(256-filter.size) + filter
    code = IupLib.IupGetFile file
    case code
    when 0, 1
      filename = file[0...file.index("\0")]
      return code, filename
    else
      return code, ''
    end
  end

  # Show a modal dialog to edit a multiline text.
  # Returns the text or nil if cancelled.
  # Assumes 2048 as maximum length, unless a larger text is input.
  #
  # text:: Initial text to display.
  # title:: Title of dialog box.
  #--
  # TODO: Crashes - double free or corruption.
  # def get_text title, text=''
  #   text += ' '*(2048-text.size)
  #   text_ptr = FFI::MemoryPointer.from_string text
  #   code = IupLib.IupGetText FFI::MemoryPointer.from_string(title), text_ptr
  #   return (code.zero? ? nil : text_ptr.read_string.force_encoding('UTF-8').strip)
  # end

  # show a list dialog.
  # returns selected item, or nil if cancel clicked.
  #--
  # TODO: Include multiple selection, change to cols/lines
  def listdialog title, items
    strptrs = []
    strptrs << nil
 
    items_ptr = FFI::MemoryPointer.new(:pointer, items.length)
    items.each_with_index do |item, i|
      items_ptr[i].put_pointer(0, FFI::MemoryPointer.from_string(item))
    end

    code = IupLib.IupListDialog 1, title, items.length, items_ptr, 1, 1, 10, nil

    return (code == -1 ? nil : items[code])
  end

  # Create a MessageDialog with given title and displayed message.
  #
  #   message "title of dialog", "text of message"
  #
  def message title, msg
    IupLib.IupMessage title, msg
  end

end
