module Iup

  # A FileDialog is a predefined dialog for selecting files or a directory.
  # The dialog can only be shown using the popup function.
  #
  # == Attributes
  #
  # allownew:: If set, indicates if non-existent filenames are accepted.
  #            Values as 'yes' / 'no' - defaults to 'no'.
  # dialogtype:: 'open' / 'save' / 'dir'
  # directory:: Initial directory
  # extfilter:: Defines filters, e.g. "Text files\|*.txt;*.doc\|Image files\|*.gif;*.jpg;*.bmp\|", 
  #             has priority over +filterinfo+ and +filter+.
  # file:: Name of file. Will be used instead of +dictionary+ if it contains 
  #        a complete path.
  # filter:: Filter to use, e.g. "*.C;*.LED;test.*".
  # filterinfo:: Information about the +filter+, e.g. "C files".
  # filterused:: n, index of filter from +extfilter+ that was used. 
  #              Index counts from 1.
  # multiplefiles:: 'no' / 'yes', set to allow selection of multiple files.
  # nochangedir:: 'yes' / 'no', if set, restores current directory after dialog is closed.
  # nooverwriteprompt:: 'no' / 'yes', if set, prompts before overwriting existing files.
  # parentdialog:: This dialog will be always in front of the parent dialog. 
  #                If the parent is minimized, this dialog is automatically minimized.
  #                *Important* Closing the parent will also close the child, but the 
  #                child dialog's CLOSE_CB method will not be called.
  # showhidden:: 'no' / 'yes', if set, shows hidden files.
  # showpreview:: 'no' / 'yes', if set, shows a preview area for file.
  # status:: <b>read-only</b> Returns an integer: 1 for new file, 0 for existing file/directory, for -1 for cancelled.
  # title:: Title text for the file dialog.
  # value:: <b>read-only</b> Name of file(s) selected.
  #
  class FileDialog < Widget

    # Creates a dialog, using the optional block to set its attributes.
    def initialize &block
      @handle = IupLib.IupFileDlg

      self.instance_eval &block if block_given?
    end

    # Shows the dialog at position x, y.
    def popup x=0, y=0
      IupLib.IupPopup @handle, x, y
    end

    # -- attributes

    define_attribute :allownew
    define_attribute :dialogtype
    define_attribute :directory
    define_attribute :extfilter
    define_attribute :file
    define_attribute :filter
    define_attribute :filterinfo
    define_attribute :filterused
    define_attribute :multiplefiles
    define_attribute :nochangedir
    define_attribute :nooverwriteprompt

    def parentdialog parent=nil # :nodoc:
      attribute_reference 'PARENTDIALOG', Dialog, parent
    end

    define_attribute :showhidden
    define_attribute :showpreview
    
    def status # :nodoc:
      status = IupLib.IupGetAttribute(@handle, 'STATUS').first
      status.to_i
    end

    define_attribute :title
    define_readonly :value
    
    # -- callbacks

    # Called when a file is selected. 
    # callback:: takes two arguments, a filename and a status.
    def file_cb callback
      unless callback.arity == 2
        raise ArgumentError, 'file_cb callback must take 2 arguments, a filename, and status'
      end
      cb = Proc.new do |ih, fn_ptr, st_ptr|
        fn = FFI::Pointer.new(fn_ptr).read_string
        st = FFI::Pointer.new(st_ptr).read_string
        callback.call fn, st
      end
      define_callback cb, 'FILE_CB', :ss_i
    end
  end
end

