module Iup

  # Loads an image from file and returns an appropriate ruby class
  def Iup.LoadImage filename
    handle = ImLib.IupLoadImage filename
    image = ImageWidget.new
    image.instance_eval do
      @handle = handle
    end
    return image
  end

  # Parent class of images provides common attributes.
  # 
  # == Attributes
  #
  # assign_handle:: <b>write-only</b> sets the name for referring to image.
  # bpp:: <b>read-only</b> Returns the number of bits per pixel.
  # channels:: <b>read-only</b> Returns the number of channels.
  # color:: index, "r g b" -> gets/sets colour index. (Used only in 8-bit images created using +Image+)
  # colour:: index, "r g b" -> gets/sets colour index. (Used only in 8-bit images created using +Image+)
  # height:: <b>read-only</b> Height of the image, in pixels.
  # hotspot:: "x:y", coordinates of spot in image used for mouse click. Default is "0:0".
  # rastersize:: <b>read-only</b> Size of the image, in pixels, value as "widthxheight".
  # wid:: <b>read-only</b> Native widget identifier.
  # width:: <b>read-only</b> Width of the image, in pixels.
  #
  class ImageWidget < Widget

    define_readonly :bpp
    define_readonly :channels
    
    def colour index, value #:nodoc:
      IupLib.IupSetAttribute @handle, index.to_s, value
    end
    
    def color index, value # :nodoc:
      colour index, value
    end

    define_readonly :height
    define_attribute :hotspot
    define_readonly :rastersize
    define_readonly :wid
    define_readonly :width

    # Saves image to given filename.
    # Optional format is one of: "BMP", "JPEG", "GIF", "TIFF", "PNG", "PNM", "PCX", "ICO", etc.
    def save filename, format = 'png'
      err = IupIm.IupSaveImage @handle, filename, format
      if err.zero?
        raise Exception, 'Could not save image as file'
      end
    end
  end

  # Image made from greyscale values. For example:
  #
  #   pixmap_x = [
  #    1,2,3,3,3,3,3,3,3,2,1, 
  #    2,1,2,3,3,3,3,3,2,1,2, 
  #    3,2,1,2,3,3,3,2,1,2,3, 
  #    3,3,2,1,2,3,2,1,2,3,3, 
  #    3,3,3,2,1,2,1,2,3,3,3, 
  #    3,3,3,3,2,1,2,3,3,3,3, 
  #    3,3,3,2,1,2,1,2,3,3,3, 
  #    3,3,2,1,2,3,2,1,2,3,3, 
  #    3,2,1,2,3,3,3,2,1,2,3, 
  #    2,1,2,3,3,3,3,3,2,1,2, 
  #    1,2,3,3,3,3,3,3,3,2,1
  #  ]
  #  
  #  img = Image.new 11, 11, pixmap_x
  #
  class Image < ImageWidget
    # Constructor creates an image from raw pixels, using 1-value per pixel.
    # width:: width of image in pixels
    # height:: height of image in pixels
    # pixels:: an array of pixel values
    # block:: an optional block to set attributes
    def initialize width, height, pixels, &block
      @handle = IupLib.IupImage width, height, IupLib.pointer_from_chars(pixels)

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end
  end

  # Image made from RGB values.
  class ImageRGB < ImageWidget
    # Constructor creates an image from raw pixels, using 3-values per pixel.
    # width:: width of image in pixels
    # height:: height of image in pixels
    # pixels:: an array of pixel values
    # block:: an optional block to set attributes
    def initialize width, height, pixels, &block
      @handle = IupLib.IupImageRGB width, height, IupLib.pointer_from_chars(pixels)

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end
  end

  # Image made from RGBA values.
  class ImageRGBA < ImageWidget
    # Constructor creates an image from raw pixels, using 4-values per pixel.
    # width:: width of image in pixels
    # height:: height of image in pixels
    # pixels:: an array of pixel values
    # block:: an optional block to set attributes
    def initialize width, height, pixels, &block
      @handle = IupLib.IupImageRGBA width, height, IupLib.pointer_from_chars(pixels)

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end
  end
end
