module Iup

  # Matrix
  #
  class Matrix < Widget
    def initialize &block
      open_controls
      @handle = ControlsLib.IupMatrix ""

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end

    # get value of cell
    def get lin, col
      IupLib.IupGetAttribute(@handle, "#{lin}:#{col}").first
    end

    # set value of cell
    def set lin, col, value
      IupLib.IupSetAttribute @handle, "#{lin}:#{col}", value
    end 

    # -- attributes

    # general attributes

    define_attribute :cursor
    define_attribute :dropimage
    define_attribute :focuscell
    define_attribute :flat
    define_attribute :flatscrollbar
    define_attribute :hidefocus
    define_attribute :hiddentextmarks
    define_attribute :hlcolor
    define_attribute :origin
    define_attribute :originoffset
    define_attribute :readonly
    define_attribute :showfillvalue
    define_attribute :togglecentered
    define_attribute :toggleimageon
    define_attribute :toggleimageoff
    define_attribute :typecolorinactive

    define_attribute :expand
    define_readonly :position
    define_attribute :rastersize
    define_readonly :screenposition
    define_attribute :tip

    # cell attributes (no redraw)

    def bgcolor_column col, colour
      IupLib.IupSetAttribute @handle, "BGCOLOR*:#{col}", colour
    end
    def bgcolor_line lin, colour
      IupLib.IupSetAttribute @handle, "BGCOLOR#{lin}:*", colour
    end
    def bgcolor_cell col, lin, colour
      IupLib.IupSetAttribute @handle, "BGCOLOR#{lin}:#{col}", colour
    end

    # size attributes
    define_attribute :limitexpand
    define_attribute :resizematrix
    define_attribute :resizedrag
    define_attribute :usetitlesize

    # column size attributes

    # RASTERWIDTHn, defines column width in pixels
    def rasterwidth n, num_pixels
      IupLib.IupSetAttribute @handle, "RASTERWIDTH#{n}", num_pixels.to_s
    end

    # WIDTHn, defines column width in SIZE units
    def width n, size
      IupLib.IupSetAttribute @handle, "WIDTH#{n}", size.to_s
    end

    define_attribute :widthdef

    # MINCOLWIDTHn, defines column width in SIZE units
    def mincolwidth n, size
      IupLib.IupSetAttribute @handle, "MINCOLWIDTH#{n}", size.to_s
    end

    # line size attributes

    # HEIGHTn, defines line height in SIZE units
    def height n, size
      IupLib.IupSetAttribute @handle, "HEIGHT#{n}", size.to_s
    end

    define_attribute :heightdef

    # RASTERHEIGHTn, defines line height in pixels
    def rasterheight n, num_pixels
      IupLib.IupSetAttribute @handle, "RASTERHEIGHT#{n}", num_pixels.to_s
    end

    # number of cells attributes
    define_writeonly :addcol
    define_writeonly :addlin
    define_writeonly :delcol
    define_writeonly :dellin
    define_attribute :numcol
    define_attribute :numcol_visible
    define_attribute :numcol_noscroll
    define_attribute :numlin
    define_attribute :numlin_visible
    define_attribute :numlin_noscroll
    define_attribute :noscrollastitle

    # action attributes

    def redraw # TODO - values
      IupLib.IupSetAttribute @handle, 'redraw', 'all'
    end

    define_writeonly :fittosize

    # canvas attributes
    define_attribute :border
    define_attribute :scrollbar

    # -- callbacks

    # Action generated when a keyboard event occurs
    # returns IUP_DEFAULT to validate the key, IUP_IGNORE to ignore, 
    # IUP_CONTINUE forwards the key, or identifier of the key to be treated
    def action_cb callback
      unless callback.arity == 5
        raise ArgumentError, 'action_cb callback must take 5 arguments: (key, lin, col, edition, value)'
      end
      cb = Proc.new do |ih, key, lin, col, edition, value|
        callback.call key, lin, col, edition, value
      end
      define_callback cb, 'ACTION_CB', :i_s
    end

    # Action generated when any mouse button is pressed over a cell.
    def click_cb callback
      unless callback.arity == 3
        raise ArgumentError, 'click_cb callback must take 3 arguments: (lin, col, status)'
      end
      cb = Proc.new do |ih, lin, col, status|
        callback.call lin, col, status
      end
      define_callback cb, 'CLICK_CB', :i_s
    end

    # Action generated when a column is interactively resized.
    def colresize_cb callback
      unless callback.arity == 1
        raise ArgumentError, 'colresize_cb callback must take 1 argument: (col)'
      end
      cb = Proc.new do |ih, col|
        callback.call col
      end
      define_callback cb, 'COLRESIZE_CB', :i_s
    end

    # Action generated when any mouse button is released over a cell.
    def release_cb callback
      unless callback.arity == 3
        raise ArgumentError, 'release_cb callback must take 3 arguments: (lin, col, status)'
      end
      cb = Proc.new do |ih, lin, col, status|
        callback.call lin, col, status
      end
      define_callback cb, 'RELEASE_CB', :i_s
    end

    # Action generated after the element size has been updated, but 
    # before the cells are refreshed.
    def resizematrix_cb callback
      unless callback.arity == 2
        raise ArgumentError, 'resizematrix_cb callback must take 2 arguments: (width, height)'
      end
      cb = Proc.new do |ih, width, height|
        callback.call width, height
      end
      define_callback cb, 'RESIZEMATRIX_CB', :i_s
    end

    # Action generated when mouse has moved over the matrix.
    def mousemove_cb callback
      unless callback.arity == 2
        raise ArgumentError, 'mousemove_cb callback must take 2 arguments: (lin, col)'
      end
      cb = Proc.new do |ih, lin, col|
        callback.call lin, col
      end
      define_callback cb, 'MOUSEMOVE_CB', :i_s
    end

    # Action generated when matrix cell is selected, matrix gets focus etc.
    def enteritem_cb callback
      unless callback.arity == 2
        raise ArgumentError, 'enteritem_cb callback must take 2 arguments: (lin, col)'
      end
      cb = Proc.new do |ih, lin, col|
        callback.call lin, col
      end
      define_callback cb, 'ENTERITEM_CB', :i_s
    end

    # Action generated when cell is no longer current cell.
    def leaveitem_cb callback
      unless callback.arity == 2
        raise ArgumentError, 'leaveitem_cb callback must take 2 arguments: (lin, col)'
      end
      cb = Proc.new do |ih, lin, col|
        callback.call lin, col
      end
      define_callback cb, 'LEAVEITEM_CB', :i_s
    end

    # Action generated when matrix is scrolled.
    def scrolltop_cb callback
      unless callback.arity == 2
        raise ArgumentError, 'scrolltop_cb callback must take 2 arguments: (lin, col)'
      end
      cb = Proc.new do |ih, lin, col|
        callback.call lin, col
      end
      define_callback cb, 'SCROLLTOP_CB', :i_s
    end


  end
end
