module Iup

  # Attributes for widgets that permit a scrollbar.
  #
  # For all these attributes, the optional argument is used to _set_ the value, 
  # else the current value is returned.
  #
  # == Attributes
  #
  # xautohide:: If set, hides horizontal scrollbar if not required. Values 'yes' / 'no'.
  # yautohide:: If set, hides vertical scrollbar if not required. Values 'yes' / 'no'.
  #
  module ScrollBarAttributes
    extend AttributeBuilders

    define_attribute :scrollbar

    # Defines the size of the horizontal thumbnail.
    # Size defaults to 0.1, and must be in range 0 < value < (xmax-xmin).
    def dx val=nil
      if val.nil?
        result = IupLib.IupGetAttribute(@handle, 'DX').first
        result.to_f
      else
        IupLib.IupSetAttribute @handle, 'DX', val.to_s
      end
    end

    # Defines the size of the vertical thumbnail.
    # Size defaults to 0.1, and must be in range 0 < value < (ymax-ymin).
    def dy val=nil
      if val.nil?
        result = IupLib.IupGetAttribute(@handle, 'DY').first
        result.to_f
      else
        IupLib.IupSetAttribute @handle, 'DY', val.to_s
      end
    end

    # Position of horizontal thumbnail.
    # Size starts at 0.0, and must be in range xmin < value < (xmax-dx).
    def posx val=nil
      if val.nil?
        result = IupLib.IupGetAttribute(@handle, 'POSX').first
        result.to_f
      else
        IupLib.IupSetAttribute @handle, 'POSX', val.to_s
      end
    end

    # Position of vertical thumbnail.
    # Size starts at 0.0, and must be in range ymin < value < (ymax-dy).
    def posy val=nil
      if val.nil?
        result = IupLib.IupGetAttribute(@handle, 'POSY').first
        result.to_f
      else
        IupLib.IupSetAttribute @handle, 'POSY', val.to_s
      end
    end

    # Minimum value of horizontal scrollbar. Starts at 0.0.
    def xmin val=nil
      if val.nil?
        result = IupLib.IupGetAttribute(@handle, 'XMIN').first
        result.to_f
      else
        IupLib.IupSetAttribute @handle, 'XMIN', val.to_s
      end
    end

    # Minimum value of vertical scrollbar. Starts at 0.0.
    def ymin val=nil
      if val.nil?
        result = IupLib.IupGetAttribute(@handle, 'YMIN').first
        result.to_f
      else
        IupLib.IupSetAttribute @handle, 'YMIN', val.to_s
      end
    end

    # Maximum value of horizontal scrollbar. Defaults to 1.0.
    def xmax val=nil
      if val.nil?
        result = IupLib.IupGetAttribute(@handle, 'XMAX').first
        result.to_f
      else
        IupLib.IupSetAttribute @handle, 'XMAX', val.to_s
      end
    end

    # Maximum value of vertical scrollbar. Defaults to 1.0.
    def ymax val=nil
      if val.nil?
        result = IupLib.IupGetAttribute(@handle, 'YMAX').first
        result.to_f
      else
        IupLib.IupSetAttribute @handle, 'YMAX', val.to_s
      end
    end
    
    # Amount thumb moves in horizontal direction. Default is dx/10.
    def linex val=nil
      if val.nil?
        result = IupLib.IupGetAttribute(@handle, 'LINEX').first
        result.to_f
      else
        IupLib.IupSetAttribute @handle, 'LINEX', val.to_s
      end
    end

    # Amount thumb moves in vertical direction. Default is dy/10.
    def liney val=nil
      if val.nil?
        result = IupLib.IupGetAttribute(@handle, 'LINEY').first
        result.to_f
      else
        IupLib.IupSetAttribute @handle, 'LINEY', val.to_s
      end
    end

    define_attribute :xautohide
    define_attribute :yautohide

    # Called when scrollbar used. 
    # Callback must take three arguments: 
    # op:: operation performed on scrollbar, see Iup
    # posx:: x position of scroll thumb (same as posx attribute)
    # posy:: y position of scroll thumb (same as posy attribute)
    def scroll_cb callback
      unless callback.arity == 3
        raise ArgumentError, 'scroll_cb callback must take 3 arguments: (op, posx, posy)'
      end
      cb = Proc.new do |ih, op, posx, posy|
        callback.call op, posx, posy
      end
      define_callback cb, 'SCROLL_CB', :iff_i
    end

  end
end
