module Iup

  # A Tree displays a hierarchy of branch and leaf nodes.
  # Each node may display some text and an optional image.
  #
  # The contents of the tree can _only_ be created after the tree has been 
  # located in a dialog, and before the Dialog#show method is called.
  # See the example below:
  #
  #   mainloop do
  #   
  #     tree = Tree.new
  #   
  #     dlg = Dialog.new tree do
  #       title 'Tree Example'
  #     end.map
  #   
  #     tree.instance_eval do
  #       title 'Figures'
  #       size '80x80'
  #       font 'Courier, Normal 10'
  #       title 'Figures'
  #       addbranch 0, '3D'
  #       addbranch 0, '2D'
  #       addleaf 1, 'trapeze'
  #       addbranch 1, 'parallelogram'
  #       addleaf 2, 'diamond'
  #       addleaf 2, 'square'
  #       addbranch 4, 'triangle'
  #       addleaf 5, 'scalenus'
  #       addleaf 5, 'isoceles'
  #       addleaf 5, 'equilateral'
  #       value 6
  #       addexpanded 'no'
  #     end 
  #   
  #     dlg.show
  #   end
  #
  # == Attributes
  #
  # addexpanded:: 'yes' / 'no', to expand branches when created.
  # addroot:: 'yes' / 'no', automatically adds an empty branch as 
  #           the first node, on creation.
  # canfocus:: Enables the control to gain focus. Values 'yes' / 'no'.
  # count:: <b>read-only</b> returns total number of nodes in the tree.
  # dragdroptree:: 'yes' / 'no'.
  # dropequaldrag:: 'yes' / 'no', if set, allows a drop node to equal drag node.
  # expand:: Allows control to fill available space in indicated direction.
  #          Values 'no' / 'horizontal' / 'vertical' / 'yes'.
  # hidebuttons:: 'yes' / 'no', to hide expand and create buttons.
  # hidelines:: 'yes' / 'no', the lines connecting nodes in hierarchy.
  # indentation:: level of indentation in pixels, defaults to 5.
  # position:: <b>read-only</b> returns position in pixels within client window
  #            as "x,y".
  # rastersize:: Size of the control, in pixels, value as "widthxheight".
  # screenposition:: <b>read-only</b> returns position in pixels on screen
  #                  as "x,y".
  # showdragdrop:: 'yes' / 'no', enables internal drag and drop of nodes.
  # showtoggle:: 'yes' / 'no' / '3state', enables use of toggles for all nodes of the tree.
  # spacing:: vertical, internal padding for each node, defaults to 3 pixels.
  # topitem:: <b>write-only</b> Positions given node id at the top of tree,
  #           or near, to make it visible.
  # value:: When retrieved, returns the identifier of the focussed node. When 
  #         given a value, moves focus appropriately: 
  #         'root' / 'last' / 'next' / 'previous' / 'pgdn' / 'pgup' 
  #
  # mark:: <b>write-only> selects a range of nodes:
  #        'start-end' / 'INVERTid' / 'block' / 'clearall' / 'markall' / 'invertall'
  # markmode:: 'single' / 'multiple', for selection of nodes.
  # markstart:: Initial node for block marking, used when mark=block.
  #
  # expandall:: Expands or contracts all nodes, values 'yes' / 'no'.
  #
  class Tree < Widget
    include DragDropAttributes 

    # Creates an instance of Tree which is set up via the _block_.
    def initialize &block
      @handle = IupLib.IupTree

      # run any provided block on instance, to set up further attributes
      self.instance_eval &block if block_given?
    end

    # -- attributes

    # --- general
    define_attribute :addexpanded
    define_attribute :addroot
    define_attribute :canfocus
    define_readonly :count
    define_attribute :dragdroptree
    define_attribute :dropequaldrag
    define_attribute :expand
    define_attribute :hidebuttons
    define_attribute :hidelines
    define_attribute :indentation
    define_readonly :position
    define_attribute :rastersize
    define_readonly :screenposition
    define_attribute :showdragdrop
    define_attribute :showtoggle
    define_attribute :spacing
    define_attribute :topitem
    define_attribute :value

    # --- nodes

    # Returns the number of immediate child of given node.
    # id:: identifier of a node
    def childcount id
      IupLib.IupGetAttribute(@handle, "CHILDCOUNT#{id}").first
    end
    
    # Controls text foreground colour: 
    # provide a 'val' to set colour, otherwise, current colour returned.
    # id:: identifier of a node
    # val:: optional "r g b" colour
    def color id, val=nil
      if val.nil?
        IupLib.IupGetAttribute(@handle, "COLOR#{id}").first
      else
        IupLib.IupSetAttribute @handle, "COLOR#{id}", val
      end
    end

    # Returns the depth of the given node.
    # id:: identifier of a node
    def depth id
      IupLib.IupGetAttribute(@handle, "DEPTH#{id}").first
    end

    # Returns kind of given node, as 'leaf' or 'branch'.
    # id:: identifier of a node
    def kind id
      IupLib.IupGetAttribute(@handle, "KIND#{id}").first
    end
    
    # Returns identifier of parent of given node.
    # id:: identifier of a node
    def parent id
      IupLib.IupGetAttribute(@handle, "PARENT#{id}").first
    end

    # Controls state as 'collapsed' or 'expanded': 
    # provide a 'val' to set state, otherwise, current state returned (nil for a leaf).
    # id:: identifier of a node
    # val:: optional value 'collapsed' or 'expanded'.
    def state id, val=nil
      if val.nil?
        IupLib.IupGetAttribute(@handle, "STATE#{id}").first
      else
        IupLib.IupSetAttribute @handle, "STATE#{id}", val
      end
    end

    # Controls title of given node: 
    # provide a 'val' to set, otherwise, current title returned.
    # id:: identifier of a node
    # val:: optional title.
    def title id, val=nil
      if val.nil?
        IupLib.IupGetAttribute(@handle, "TITLE#{id}").first
      else
        IupLib.IupSetAttribute @handle, "TITLE#{id}", val
      end
    end

    # Controls font of node title: 
    # provide a 'val' to set font, otherwise, current font returned.
    # id:: identifier of a node
    # val:: optional font description.
    def titlefont id, val=nil
      if val.nil?
        IupLib.IupGetAttribute(@handle, "TITLEFONT#{id}").first
      else
        IupLib.IupSetAttribute @handle, "TITLEFONT#{id}", val
      end
    end

    # Controls toggle state of given node: 
    # provide a 'val' to set state, otherwise, current state returned.
    # id:: identifier of a node
    # val:: optional state as 'on' / 'off' / 'notdef'
    def togglevalue id, val=nil
      if val.nil?
        IupLib.IupGetAttribute(@handle, "TOGGLEVALUE#{id}").first
      else
        IupLib.IupSetAttribute @handle, "TOGGLEVALUE#{id}", val
      end
    end

    # Controls toggle visibility of given node: 
    # provide a 'val' to set visibility, otherwise, current visibility returned.
    # id:: identifier of a node
    # val:: optional visibility as 'yes' / 'no'.
    def togglevisible id, val=nil
      if val.nil?
        IupLib.IupGetAttribute(@handle, "TOGGLEVISIBLE#{id}").first
      else
        IupLib.IupSetAttribute @handle, "TOGGLEVISIBLE#{id}", val
      end
    end

    # Returns total number of children for given node.
    # id:: identifier of a node
    def totalchildcount id
      IupLib.IupGetAttribute(@handle, "TOTALCHILDCOUNT#{id}").first
    end

    # Controls user data associated with given node: 
    # provide a 'val' to set userdata, otherwise, current userdata returned.
    # id:: identifier of a node
    # val:: optional userdata to associate with given node.
    def userdata id, val=nil
      if val.nil?
        IupLib.IupGetAttribute(@handle, "USERDATA#{id}").first
      else
        IupLib.IupSetAttribute @handle, "USERDATA#{id}", val
      end
    end

    # --- images

    # Sets image to use for given node.
    # id:: identifier of a node
    # img:: image name or reference
    def image id, img
      attribute_reference "IMAGE#{id}", ImageWidget, img
    end

    # Sets image to use for given node, for expanded branches.
    # id:: identifier of a node
    # img:: image name or reference
    def imageexpanded id, img
      attribute_reference "IMAGEEXPANDED#{id}", ImageWidget, img
    end

    # If given, sets image to be used for all leaf nodes, 
    # otherwise returns current value.
    # img:: optional image name or reference
    def imageleaf img=nil
      attribute_reference "IMAGELEAF", ImageWidget, img
    end

    # If given, sets image to be used for all collapsed branches, 
    # otherwise returns current value.
    # img:: optional image name or reference
    def imagebranchcollapsed img=nil
      attribute_reference "IMAGEBRANCHCOLLAPSED", ImageWidget, img
    end

    # If given, sets image to be used for all expanded branches, 
    # otherwise returns current value.
    # img:: optional image name or reference
    def imagebranchexpanded img=nil
      attribute_reference "IMAGEBRANCHEXPANDED", ImageWidget, img
    end

    # --- marks
    define_writeonly :mark

    # Selection state of specific node returned or, if val given, set.
    # id:: identifier of a node
    # val:: optional 'yes' / 'no'
    def marked id, val=nil
      if val.nil?
        IupLib.IupGetAttribute(@handle, "MARKED#{id}").first
      else
        IupLib.IupSetAttribute @handle, "MARKED#{id}", val
      end
    end

    # Sets/gets selection indices of all marked nodes, when 
    # markmode=multiple.
    # val:: optional [ids]
    def markednodes val=nil
      if val.nil?
        mns = IupLib.IupGetAttribute(@handle, 'MARKEDNODES').first
        result = []
        mns.split('').each_with_index do |item, index|
          result << index+1 if item == '+'
        end
        return result
      else
        result = ""
        count.to_i.times do |i|
          if val.include?(i+1)
            result << '+'
          else
            result << '-'
          end
        end
        IupLib.IupSetAttribute @handle, 'MARKEDNODES', result
      end
    end
    
    define_attribute :markmode
    define_attribute :markstart

    # --- hierarchy

    # Adds new branch after specified node.
    # id:: identifier of a node
    # val:: text label
    def addbranch id, val
      IupLib.IupSetAttribute @handle, "ADDBRANCH#{id}", val.to_s
    end

    # Adds new leaf after specified node.
    # id:: identifier of a node
    # val:: text label
    def addleaf id, val
      IupLib.IupSetAttribute @handle, "ADDLEAF#{id}", val.to_s
    end

    # Copies source node and its children to target.
    # source_id:: identifier of a node
    # target_id:: identifier of a node
    def copynode source_id, target_id
      IupLib.IupSetAttribute @handle, "COPYNODE#{id}", target_id.to_s
    end

    # Delete one or more nodes, depending on val:
    # * 'all': ignores id and deletes all nodes in tree, including root.
    # * 'selected': deletes selected node and its children.
    # * 'children': deleted only the children of selected node.
    # * 'marked': ignores id and deletes all selected node.
    #
    # id:: identifier of a node
    # val:: one of above values
    def delnode id, val
      IupLib.IupSetAttribute @handle, "DELNODE#{id}", val.to_s
    end

    define_attribute :expandall

    # Inserts new branch after specified node, preserving depth.
    # id:: identifier of a node
    # val:: text label
    def insertbranch id, val
      IupLib.IupSetAttribute @handle, "INSERTBRANCH#{id}", val.to_s
    end

    # Inserts new leaf after specified node, preserving depth.
    # id:: identifier of a node
    # val:: text label
    def insertleaf id, val
      IupLib.IupSetAttribute @handle, "INSERTLEAF#{id}", val.to_s
    end
    
    # Moves source to target, as a new child or sibling.
    # source_id:: identifier of a node
    # target_id:: identifier of a node
    def movenode source_id, target_id
      IupLib.IupSetAttribute @handle, "MOVENODE#{id}", target_id.to_s
    end

    # -- callbacks

    # Action generated when a node is selected or deselected. 
    def selection_cb callback
      unless callback.arity == 2
        raise ArgumentError, 'selection_cb callback must take 2 arguments: node_id, status'
      end
      cb = Proc.new do |ih, id, status|
        callback.call id, status
      end
      define_callback cb, 'SELECTION_CB', :ii_i
    end

    # Action generated when a branch is expanded.
    def branchopen_cb callback
      unless callback.arity == 1
        raise ArgumentError, 'branchopen_cb callback must take 1 argument: node_id'
      end
      cb = Proc.new do |ih, id|
        callback.call id
      end
      define_callback cb, 'BRANCHOPEN_CB', :i_i
    end

    # ction generated when a branch is collapsed.
    def branchclose_cb callback
      unless callback.arity == 1
        raise ArgumentError, 'branchclose_cb callback must take 1 argument: node_id'
      end
      cb = Proc.new do |ih, id|
        callback.call id
      end
      define_callback cb, 'BRANCHCLOSE_CB', :i_i
    end

    # Action generated when a leaf is to be executed
    def executeleaf_cb callback
      unless callback.arity == 1
        raise ArgumentError, 'executeleaf_cb callback must take 1 argument: node_id'
      end
      cb = Proc.new do |ih, id|
        callback.call id
      end
      define_callback cb, 'EXECUTELEAF_CB', :i_i
    end

    # Action generated when the right mouse button is pressed over a node.
    def rightclick_cb callback
      unless callback.arity == 1
        raise ArgumentError, 'rightclick_cb callback must take 1 argument: node_id'
      end
      cb = Proc.new do |ih, id|
        callback.call id
      end
      define_callback cb, 'RIGHTCLICK_CB', :i_i
    end

    # Action generated when the toggle's state was changed.
    def togglevalue_cb callback
      unless callback.arity == 2
        raise ArgumentError, 'togglevalue_cb callback must take 2 arguments: node_id, status'
      end
      cb = Proc.new do |ih, id, status|
        callback.call id, status
      end
      define_callback cb, 'TOGGLEVALUE_CB', :ii_i
    end 

    include ButtonCallback

    # Action generated when an internal drag & drop is executed. Only active if SHOWDRAGDROP=YES.
    # dragdrop_cb takes a callback which accepts 4 arguments (drag_id, drop_id, isshift, iscontrol)
    # drag_id:: is an integer index of dragged item
    # drop_id:: is an integer index of drop location
    # isshift:: boolean flag for if shift key held
    # iscontrol:: boolean flag for if control key held
    # callback should return Iup::CONTINUE for item to be moved/copied.
    def dragdrop_cb callback
      unless callback.arity == 4
        raise ArgumentError, 'dragdrop_cb callback must take 4 arguments: (drag_id, drop_id, isshift, iscontrol)'
      end
      cb = Proc.new do |ih, drag_id, drop_id, isshift, iscontrol|
        callback.call drag_id.to_i, drop_id.to_i, (isshift != 0), (iscontrol != 0)
      end
      define_callback cb, 'DRAGDROP_CB', :iiii_i
    end

    # Action generated when the mouse is moved.
    # Callback takes 3 arguments: (x, y, state)
    # x:: x position of mouse
    # y:: y position of mouse
    # state:: status of mouse buttons and certain keyboard keys at the moment the event was generated.
    #--
    # TODO: include functions, as in button_cb
    #
    def motion_cb callback
      unless callback.arity == 3
        raise ArgumentError, 'motion_cb callback must take 3 arguments: (x, y, state)'
      end
      cb = Proc.new do |ih, x, y, state|
        callback.call x, y, state
      end
      define_callback cb, 'MOTION_CB', :iis_i
    end
  end
end
