module Iup

  # Parent class for controls in the library.
  #
  class Widget
    extend AttributeBuilders
    include CommonAttributes
    include CallbackSetter

    # Unique reference to widget.
    attr_reader :handle

    # The C handle name for this widget
    def assign_handle name
      IupLib.IupSetHandle name, @handle
    end

    @@controlslib = false

    # Ensure the controls library is opened
    def open_controls
      unless @@controlslib
        ControlsLib.IupControlsOpen
        @@controlslib = true
      end
    end

    # -- common callbacks

    # Called right after an element is mapped and its layout updated
    def map_cb callback
      unless callback.arity.zero?
        raise ArgumentError, 'map_cb callback must take 0 arguments'
      end
      cb = Proc.new do |ih|
        callback.call
      end
      define_callback cb, 'MAP_CB', :plain
    end

    # Called right before an element is unmapped
    def unmap_cb callback
      unless callback.arity.zero?
        raise ArgumentError, 'unmap_cb callback must take 0 arguments'
      end
      cb = Proc.new do |ih|
        callback.call
      end
      define_callback cb, 'UNMAP_CB', :plain
    end

    # Action generated when an element is given keyboard focus
    def getfocus_cb callback
      unless callback.arity.zero?
        raise ArgumentError, 'getfocus_cb callback must take 0 arguments'
      end
      cb = Proc.new do |ih|
        callback.call
      end
      define_callback cb, 'GETFOCUS_CB', :plain
    end

    # Action generated when an element loses keyboard focus
    def killfocus_cb callback
      unless callback.arity.zero?
        raise ArgumentError, 'killfocus_cb callback must take 0 arguments'
      end
      cb = Proc.new do |ih|
        callback.call
      end
      define_callback cb, 'KILLFOCUS_CB', :plain
    end

    # Action generated when the mouse enters the element
    def enterwindow_cb callback
      unless callback.arity.zero?
        raise ArgumentError, 'enterwindow_cb callback must take 0 arguments'
      end
      cb = Proc.new do |ih|
        callback.call
      end
      define_callback cb, 'ENTERWINDOW_CB', :plain
    end

    # Action generated when the mouse leaves the element
    def leavewindow_cb callback
      unless callback.arity.zero?
        raise ArgumentError, 'leavewindow_cb callback must take 0 arguments'
      end
      cb = Proc.new do |ih|
        callback.call
      end
      define_callback cb, 'LEAVEWINDOW_CB', :plain
    end
    
    # Action generated when a keyboard event occurs.
    # k_any takes a 1-argument callback, the argument 
    # being the character that is pressed.
    def k_any callback
      unless callback.arity == 1
        raise ArgumentError, 'k_any callback must take 1 argument, a character'
      end
      cb = Proc.new do |ih, c|
        callback.call c
      end
      define_callback cb, 'K_ANY', :i_i
    end

    # Action generated when the user presses F1 at a control
    def help_cb callback
      unless callback.arity.zero?
        raise ArgumentError, 'help_cb callback must take 0 arguments'
      end
      cb = Proc.new do |ih|
        callback.call
      end
      define_callback cb, 'HELP_CB', :plain_v
    end

    private

    # given an array of widgets, 
    # convert into format widget1, :pointer, widget2, ... , :pointer, nil
    def widget_list widgets
      result = []

      widgets.each do |widget|
        result << widget.handle
        result << :pointer
      end

      result << nil

      return result
    end
  end
end
